{% load i18n %}
{% load static %}
<div id="ow-chart-inner-container">
    <script type="text/javascript" src="{% static 'monitoring/js/lib/daterangepicker.min.js' %}"></script>
    <span id="monitoring-timeseries-api-url" data-value="{{ api_url }}"></span>
    <span id="monitoring-timeseries-original-key" data-value="{{ original.key }}"></span>
    <span id="monitoring-timeseries-default-time" data-value="{{ default_time }}"></span>
    <div id="ow-chart-utils">
        <span>
            <a id="daterangepicker-widget">
                <span class="button" ></span>
            </a>
            <a id="ow-chart-export" class="button export">{% trans 'export data' %}</a>
        </span>
        <span style="display:none";>
            <select name="org-selector" id="org-selector">
                <option></option>
            </select>
        </span>
    </div>
    <div id="chart-loading-overlay"><div class="ow-loading-spinner"></div></div>
    <div id="ow-chart-contents"></div>
    <div id="ow-chart-fallback" class="form-row">
        {% trans 'Insufficient data for selected time period' %}.
    </div>
    {% if chart_quick_links %}
        {{ chart_quick_links|json_script:"monitoring-chart-quick-links" }}
    {% endif %}
</div>
