# Generated by Django 3.1.2 on 2020-12-20 06:15


from django.db import migrations


class Migration(migrations.Migration):
    dependencies = [
        ('device_monitoring', '0001_squashed_0002_devicemonitoring'),
    ]

    operations = [
        # NOTE: Usage of "Monitoring Template" is deprecated in favour of
        # "openwisp-openwisp-monitoring" package. See
        # https://github.com/openwisp/openwrt-openwisp-monitoring#openwrt-openwisp-monitoring
        migrations.RunPython(
            migrations.RunPython.noop, reverse_code=migrations.RunPython.noop
        )
    ]
