# Generated by Django 4.0.4 on 2022-08-19 11:28

from django.db import migrations

from . import assign_check_inline_permissions_to_groups


class Migration(migrations.Migration):
    dependencies = [
        ('check', '0008_alter_check_options'),
    ]

    operations = [
        migrations.RunPython(
            assign_check_inline_permissions_to_groups,
            reverse_code=migrations.RunPython.noop,
        ),
    ]
