#ow-chart-fallback {
  text-align: center;
}
#ow-chart-utils {
  display: flex;
  justify-content: space-between;
}
#ow-chart-utils .select2-container {
  min-width: 150px !important;
  width: unset !important;
}
#ow-chart-utils span.select2-selection {
  min-height: 27px;
  min-width: 200px;
  height: 27px;
}
#ow-chart-utils .select2-selection__rendered {
  line-height: 27px;
  height: 27px;
}
#ow-chart-utils .select2-selection__arrow {
  top: 0;
  height: 27px;
}
#ow-chart-utils>span {
  display: flex;
  justify-content: space-between;
  width: 100%;
}
#ow-chart-utils .button {
  font-size: 14px;
  padding: 5px 10px;
}
#ow-chart-export {
  margin-left: 10px;
}
#ow-chart-fallback {
  display: none;
  border: 0 none;
}
#ow-chart-contents > div {
  position: relative;
}
.chart-help {
  display: inline-block;
  width: 22px;
  height: 21px;
  text-indent: -9999px;
  background: url(../../admin/img/icon-unknown.svg) 0 0 no-repeat;
  background-size: 80%;
  background-position: 50% 50%;
  opacity: 0.6;
  cursor: pointer;
  margin-left: 3px;
}
.chart-heading {
  margin: 0;
  font-size: 2em;
  text-align: center;
  font-size: 1.22em;
  font-weight: normal;
  color: #434b62;
}
.ow-chart {
  margin-top: 1.5em;
  padding-top: 1.5em;
  border-top: 1px dotted rgba(0, 0, 0, 0.2);
  text-align: center;
}
.ow-chart:first-child {
  border: 0 none;
  padding-top: 0;
  margin-top: 2.25em;
}
.ow-chart .summary {
  text-align: right;
  margin: 0;
}
.ow-chart .circle {
  display: inline-block;
  float: none;
  margin: 0em 0.15em 0 0.15em;
  background: rgba(0, 0, 0, 0.125);
}
#ow-chart-contents .percircle:hover > span {
  transform: scale(1.15) translateY(-50%);
  letter-spacing: -0.3px;
}
.percircle:hover:after {
  transform: scale(1.05);
}
.percircle.small {
  font-size: 90px;
}
.ow-chart .custom-legend {
  margin: 0 0 1.5em;
}
.ow-chart .legend {
  display: inline-block;
  margin: 0 0.5em;
  font-size: 13px;
  cursor: default;
}
.ow-chart .legend span {
  display: inline-block;
  width: 10px;
  height: 10px;
  margin-right: 2px;
}
#ow-chart-contents .circle > span {
  font-size: 0.15em;
  word-spacing: -0.75px;
  color: rgba(0, 0, 0, 0.6) !important;
  font-size: 13px;
  font-weight: bold;
}
#ow-chart-contents b {
  background-color: #000 !important;
}
.chart-heading {
  margin: 0;
  font-size: 2em;
  text-align: center;
  font-size: 1.22em;
  font-weight: normal;
  color: #434b62;
}
#chart-loading-overlay {
  width: 100%;
  margin: 25px auto;
  display: none;
}
.tooltip {
  position: absolute;
  background-color: rgba(0, 0, 0, 0.7);
  color: #fff;
  padding: 15px !important;
  max-width: 500px;
  line-height: 1.6em;
  border-radius: 4px;
  top: 40%;
  left: 50%;
  transform: translate(-50%, -50%);
  text-align: center;
  display: none;
  font-size: 1.2em;
}
@media (max-width: 768px) {
  #ow-chart-utils {
    display: block;
  }
  #ow-chart-utils > span + span {
    margin: 5px 0px;
    justify-content: end;
  }
}
