.health-deactivated {
  color: #000;
}
.health-unknown {
  color: grey;
}
.health-ok {
  color: green;
}
.health-problem {
  color: orange;
}
.health-critical {
  color: red;
}
.health-status {
  font-weight: bold;
  text-transform: uppercase;
}
.map-detail .paginator a {
  padding: 3px 7px;
  margin: 0 3px;
  background-color: #464646;
  color: #fff;
  text-decoration: none;
  cursor: pointer;
}
.map-detail .paginator a:hover {
  background-color: #000000;
  color: #fff;
}
#device-map-container {
  margin: 50px auto;
  position: relative;
  height: 300px;
  width: 90%;
  border: 1px solid rgba(0, 0, 0, 0.3);
}
#device-map-container .ow-loading-spinner {
  position: absolute;
  z-index: 9999;
  top: 41%;
  left: 49%;
}
#device-map-container .leaflet-interactive:focus {
  outline: none;
}
#leaflet-config {
  display: none;
}
.no-data {
  display: none;
  position: absolute;
  z-index: 99;
  top: 0;
  left: 0;
  width: 100%;
  text-align: center;
  padding-top: 115px;
}
.no-data a.button {
  color: #fff;
  padding: 5px 10px;
  font-weight: bold;
  text-decoration: none !important;
}
