# Generated by Django 3.2.18 on 2023-04-04 10:25

from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ('device_monitoring', '0005_add_group_permissions'),
    ]

    operations = [
        migrations.AlterField(
            model_name='wificlient',
            name='ht',
            field=models.BooleanField(
                blank=True, default=None, null=True, verbose_name='HT'
            ),
        ),
        migrations.AlterField(
            model_name='wificlient',
            name='vht',
            field=models.BooleanField(
                blank=True, default=None, null=True, verbose_name='VHT'
            ),
        ),
    ]
