# Generated by Django 3.2.12 on 2022-04-23 17:50

import collections

import jsonfield.fields
from django.db import migrations


class Migration(migrations.Migration):
    dependencies = [
        ('contenttypes', '0002_remove_content_type_name'),
        ('monitoring', '0003_populate_metric_is_healthy_tolerance'),
    ]

    operations = [
        migrations.AddField(
            model_name='metric',
            name='main_tags',
            field=jsonfield.fields.JSONField(
                blank=True,
                db_index=True,
                default=dict,
                dump_kwargs={'indent': 4},
                load_kwargs={'object_pairs_hook': collections.OrderedDict},
                verbose_name='main tags',
            ),
        ),
        migrations.AddField(
            model_name='metric',
            name='extra_tags',
            field=jsonfield.fields.JSONField(
                blank=True,
                default=dict,
                dump_kwargs={'indent': 4},
                load_kwargs={'object_pairs_hook': collections.OrderedDict},
                verbose_name='extra tags',
            ),
        ),
        migrations.AlterUniqueTogether(
            name='metric',
            unique_together={
                ('key', 'field_name', 'content_type', 'object_id', 'main_tags')
            },
        ),
    ]
