# Generated by Django 3.2.14 on 2022-08-12 11:04

from django.db import migrations


class Migration(migrations.Migration):
    dependencies = [
        ('monitoring', '0008_create_general_metrics'),
    ]

    operations = [
        migrations.AlterModelOptions(
            name='alertsettings',
            options={
                'permissions': (
                    ('add_alertsettings_inline', 'Can add Alert settings inline'),
                    ('change_alertsettings_inline', 'Can change Alert settings inline'),
                    ('delete_alertsettings_inline', 'Can delete Alert settings inline'),
                    ('view_alertsettings_inline', 'Can view Alert settings inline'),
                ),
                'verbose_name': 'Alert settings',
                'verbose_name_plural': 'Alert settings',
            },
        ),
    ]
