# Generated by Django 4.0.1 on 2022-01-10 11:27

from django.db import migrations


class Migration(migrations.Migration):
    dependencies = [
        ('check', '0005_create_config_applied'),
    ]

    operations = [
        migrations.RenameField(
            model_name='check',
            old_name='check',
            new_name='check_type',
        ),
    ]
