# Generated by Django 3.0.3 on 2020-07-30 15:54

from django.db import migrations, models

from ..settings import CHECK_CLASSES


class Migration(migrations.Migration):
    dependencies = [('check', '0003_create_ping')]

    operations = [
        migrations.RenameField(
            model_name='check', old_name='active', new_name='is_active'
        ),
        migrations.AlterField(
            model_name='check',
            name='is_active',
            field=models.BooleanField(
                db_index=True,
                default=True,
                help_text='whether the check should be run, related metrics collected and alerts sent',
                verbose_name='active',
            ),
        ),
        migrations.AlterField(
            model_name='check',
            name='check',
            field=models.CharField(
                choices=CHECK_CLASSES,
                db_index=True,
                max_length=128,
                verbose_name='check type',
            ),
        ),
    ]
