# Generated by Django 4.0.4 on 2022-05-20 09:06

from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ('monitoring', '0006_migrate_influxdb_structure'),
    ]

    operations = [
        migrations.AlterField(
            model_name='metric',
            name='object_id',
            field=models.CharField(blank=True, db_index=True, max_length=36, null=True),
        ),
    ]
