{% load leaflet_tags i18n %}
{% load i18n %}
{% load static %}
<script type="text/javascript">
  window._owGeoMapConfig = {
    geoJsonUrl: '{{ monitoring_location_geojson_url }}',
    locationDeviceUrl: '{{ monitoring_device_list_url }}?page_size=5'
  }
</script>
<script type="text/javascript" src={%static 'monitoring/js/lib/netjsongraph.min.js' %}></script>
<script type="text/javascript" src={%static 'monitoring/js/lib/leaflet.fullscreen.min.js' %}></script>
<div id='leaflet-config'>
  {% autoescape on %}{% leaflet_json_config %}{% endautoescape %}
</div>
<div id="device-map-container">
  <div class="no-data">
    <p>{% trans 'No map data to show' %}.</p>
    <p><a class="button submit" href="#close">Close</a></p>
  </div>
  <div class="ow-loading-spinner"></div>
</div>
