# Generated by Django 4.0.4 on 2022-06-03 11:36

from django.db import migrations


class Migration(migrations.Migration):
    dependencies = [
        ('device_monitoring', '0007_add_wificlient_field_he'),
    ]

    operations = [
        migrations.AlterModelOptions(
            name='wificlient',
            options={'ordering': ('-created',), 'verbose_name': 'WiFi Client'},
        ),
    ]
